@@ -23,6 +23,7 @@ as needed to provision shares.
 import re
 
 from oslo_log import log
+from oslo_serialization import jsonutils
 from oslo_utils import excutils
 
 from manila import exception
@@ -67,21 +68,23 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
             check_for_setup_error())
 
     @na_utils.trace
-    def _get_vserver(self, share_server=None):
+    def _get_vserver(self, share_server=None, vserver_name=None):
 
-        if not share_server:
+        if share_server:
+            backend_details = share_server.get('backend_details')
+            vserver = backend_details.get(
+                'vserver_name') if backend_details else None
+
+            if not vserver:
+                msg = _('Vserver name is absent in backend details. Please '
+                        'check whether Vserver was created properly.')
+                raise exception.VserverNotSpecified(msg)
+        elif vserver_name:
+            vserver = vserver_name
+        else:
             msg = _('Share server not provided')
             raise exception.InvalidInput(reason=msg)
 
-        backend_details = share_server.get('backend_details')
-        vserver = backend_details.get(
-            'vserver_name') if backend_details else None
-
-        if not vserver:
-            msg = _('Vserver name is absent in backend details. Please '
-                    'check whether Vserver was created properly.')
-            raise exception.VserverNotSpecified(msg)
-
         if not self._client.vserver_exists(vserver):
             raise exception.VserverNotFound(vserver=vserver)
 
@@ -119,6 +122,9 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         """Creates and configures new Vserver."""
 
         vlan = network_info['segmentation_id']
+        ports = {}
+        for network_allocation in network_info['network_allocations']:
+            ports[network_allocation['id']] = network_allocation['ip_address']
 
         @utils.synchronized('netapp-VLAN-%s' % vlan, external=True)
         def setup_server_with_lock():
@@ -126,7 +132,10 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
             self._validate_network_type(network_info)
 
             vserver_name = self._get_vserver_name(network_info['server_id'])
-            server_details = {'vserver_name': vserver_name}
+            server_details = {
+                'vserver_name': vserver_name,
+                'ports': jsonutils.dumps(ports)
+            }
 
             try:
                 self._create_vserver(vserver_name, network_info)
@@ -159,7 +168,14 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
             msg = _('Vserver %s already exists.')
             raise exception.NetAppException(msg % vserver_name)
 
-        ipspace_name = self._create_ipspace(network_info)
+        # NOTE(lseki): If there's already an ipspace created for the same VLAN
+        # port, reuse it. It will be named after the previously created share
+        # server's neutron subnet id.
+        node_name = self._client.list_cluster_nodes()[0]
+        port = self._get_node_data_port(node_name)
+        vlan = network_info['segmentation_id']
+        ipspace_name = self._client.get_ipspace_name_for_vlan_port(
+            node_name, port, vlan) or self._create_ipspace(network_info)
 
         LOG.debug('Vserver %s does not exist, creating.', vserver_name)
         self._client.create_vserver(
@@ -196,8 +212,11 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.error("Failed to configure Vserver.")
+                # NOTE(dviroel): At this point, the lock was already acquired
+                # by the caller of _create_vserver.
                 self._delete_vserver(vserver_name,
-                                     security_services=security_services)
+                                     security_services=security_services,
+                                     needs_lock=False)
 
     def _get_valid_ipspace_name(self, network_id):
         """Get IPspace name according to network id."""
@@ -222,8 +241,7 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
             return client_cmode.DEFAULT_IPSPACE
 
         ipspace_name = self._get_valid_ipspace_name(ipspace_id)
-        if not self._client.ipspace_exists(ipspace_name):
-            self._client.create_ipspace(ipspace_name)
+        self._client.create_ipspace(ipspace_name)
 
         return ipspace_name
 
@@ -342,7 +360,8 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         self._delete_vserver(vserver, security_services=security_services)
 
     @na_utils.trace
-    def _delete_vserver(self, vserver, security_services=None):
+    def _delete_vserver(self, vserver, security_services=None,
+                        needs_lock=True):
         """Delete a Vserver plus IPspace and security services as needed."""
 
         ipspace_name = self._client.get_vserver_ipspace(vserver)
@@ -357,10 +376,13 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
                 interfaces_on_vlans.append(interface)
                 vlans.append(interface['home-port'])
 
-        vlans = '-'.join(sorted(set(vlans))) if vlans else None
+        if vlans:
+            vlans = '-'.join(sorted(set(vlans))) if vlans else None
+            vlan_id = vlans.split('-')[-1]
+        else:
+            vlan_id = None
 
-        @utils.synchronized('netapp-VLANs-%s' % vlans, external=True)
-        def _delete_vserver_with_lock():
+        def _delete_vserver_without_lock():
             self._client.delete_vserver(vserver,
                                         vserver_client,
                                         security_services=security_services)
@@ -371,7 +393,14 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
 
             self._delete_vserver_vlans(interfaces_on_vlans)
 
-        return _delete_vserver_with_lock()
+        @utils.synchronized('netapp-VLAN-%s' % vlan_id, external=True)
+        def _delete_vserver_with_lock():
+            _delete_vserver_without_lock()
+
+        if needs_lock:
+            return _delete_vserver_with_lock()
+        else:
+            return _delete_vserver_without_lock()
 
     @na_utils.trace
     def _delete_vserver_vlans(self, network_interfaces_on_vlans):
@@ -391,3 +420,37 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         if options['ipv6-enabled']:
             versions.append(6)
         return versions
+
+    def manage_server(self, context, share_server, identifier, driver_options):
+        """Manages a vserver by renaming it and returning backend_details."""
+        new_vserver_name = self._get_vserver_name(share_server['id'])
+        old_vserver_name = self._get_correct_vserver_old_name(identifier)
+
+        if new_vserver_name != old_vserver_name:
+            self._client.rename_vserver(old_vserver_name, new_vserver_name)
+
+        backend_details = {'vserver_name': new_vserver_name}
+        return new_vserver_name, backend_details
+
+    def unmanage_server(self, server_details, security_services=None):
+        pass
+
+    def get_share_server_network_info(
+            self, context, share_server, identifier, driver_options):
+        """Returns a list of IPs for each vserver network interface."""
+        vserver_name = self._get_correct_vserver_old_name(identifier)
+
+        vserver, vserver_client = self._get_vserver(vserver_name=vserver_name)
+
+        interfaces = vserver_client.get_network_interfaces()
+        allocations = []
+        for lif in interfaces:
+            allocations.append(lif['address'])
+        return allocations
+
+    def _get_correct_vserver_old_name(self, identifier):
+
+        # In case vserver_name includes the template, we check and add it here
+        if not self._client.vserver_exists(identifier):
+            return self._get_vserver_name(identifier)
+        return identifier
